/**
 * THINGS/THINGER 2009
 * Copyright Erich P Gatejen (c) 2001 through 2009  ALL RIGHTS RESERVED
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package things.data.processing;

/**
 * These tools are used by the processors.  They aren't meant for common use.
 * <p>
 * @author Erich P. Gatejen
 * @version 1.0
 * <p>
 * <i>Version History</i>
 * <pre>
 * EPG - Initial - 6 MAR 02
 * EPG - Add hex data - 10 OCT 06
 * </pre> 
 */
public class LexicalTool {

	/**
	 * Get character name.
	 * @param theCharacter is the character to evaluate.
	 * @return the name for the character.
	 */
	public final static int getName(int theCharacter) {
		try {
			return LEXICAL_MAP[theCharacter][LEXICAL_MAP_NAME];
		} catch (Throwable t) {
		}
		return BAD;
	}
	
	/**
	 * Get 822 type.
	 * @param theCharacter is the character to evaluate.
	 * @return the type for the character.
	 */
	public final static int get822Type(int theCharacter) {
		try {
			return LEXICAL_MAP[theCharacter][LEXICAL_MAP_822_TYPE];
		} catch (Throwable t) {
		}
		return BAD;
	}
	
	/**
	 * Get 822 type.
	 * @param theCharacter is the character to evaluate.
	 * @return the type for the character.
	 */
	public final static int getDNSType(int theCharacter) {
		try {
			return LEXICAL_MAP[theCharacter][LEXICAL_MAP_DNS_TYPE];
		} catch (Throwable t) {
		}
		return BAD;		
	}
	
	/**
	 * Get 822 Headername type.
	 * @param theCharacter is the character to evaluate.
	 * @return the type for the character.
	 */
	public final static int get822HeadernameType(int theCharacter) {
		try {
			return LEXICAL_MAP[theCharacter][LEXICAL_MAP_822_HEADERNAME];
		} catch (Throwable t) {
		}
		return BAD;		
	}
	
	
	/**
	 * Get 822 Headername type.  Allow dollar.  Some oddball systems let dollar be a header character.  This adds condition check that will make it slower than the regular version of this method.
	 * @param theCharacter is the character to evaluate.
	 * @return the type for the character.
	 */
	public final static int get822HeadernameTypeWithDollar(int theCharacter) {
		try {
			if (theCharacter==SPECIAL_DOLLAR) return ALLOWED;
			return LEXICAL_MAP[theCharacter][LEXICAL_MAP_822_HEADERNAME];
		} catch (Throwable t) {
		}
		return BAD;		
	}
	/**
	 * Get character classification.
	 * @param theCharacter is the character to evaluate.
	 * @return the classification for the character.
	 */
	public final static int getClassification(int theCharacter) {
		try {
			return LEXICAL_MAP[theCharacter][LEXICAL_MAP_CLASSIFICATION];
		} catch (Throwable t) {
		}
		return BAD;		
	}
	
	/**
	 * Get character URI type.
	 * @param theCharacter is the character to evaluate.
	 * @return the URI type for the character.
	 */
	public final static int getURIType(int theCharacter) {
		try {
			return LEXICAL_MAP[theCharacter][LEXICAL_MAP_URI_TYPE];
		} catch (Throwable t) {
		}
		return BAD;		
	}
	
	/**
	 * Get hex value for the character.  If it isn't a hex character, it'll return a negative number.
	 * @param theCharacter is the character to evaluate.
	 * @return the hex value or a negative number
	 */
	public final static int getHexValue(int theCharacter) {
		try {
			return LEXICAL_MAP[theCharacter][LEXICAL_MAP_HEXVALUE];
		} catch (Throwable t) {
		}
		return BAD;		
	}
	
	/**
	 * Get URL form encoding character type.
	 * @param theCharacter is the character to evaluate.
	 * @return the URL form encoding type for the character.
	 */
	public final static int getURLFType(int theCharacter) {
		try {
			return LEXICAL_MAP[theCharacter][LEXICAL_MAP_URLF_TYPE];
		} catch (Throwable t) {
		}
		return BAD;		
	}
	
	/**
	 * Get the uppercase version of this character.  It'll throw an exception if the character isn't alpha.
	 * @param theCharacter to get the uppercase version.
	 * @return the character.
	 */
	public final static int getUpper(int theCharacter) throws Exception {
		if ((theCharacter <VALUE_ASCII_LOW_BOTTOM) ||(theCharacter >VALUE_ASCII_HIGH_TOP) ||(LEXICAL_MAP[theCharacter][LEXICAL_MAP_CLASSIFICATION] != CLASS_ALPHA)) throw new Exception("Cannot give the uppercase of a non-alpha.");
		if (theCharacter > 'Z') return theCharacter - 32; 
		return theCharacter;  // already upper
	}
	
	/**
	 * Get the lowercase version of this character.  It'll throw an exception if the character isn't alpha.
	 * @param theCharacter to get the lowercase version.
	 * @return the character.
	 */
	public final static int getLower(int theCharacter) throws Exception {
		if ((theCharacter <VALUE_ASCII_LOW_BOTTOM) ||(theCharacter >VALUE_ASCII_HIGH_TOP) ||(LEXICAL_MAP[theCharacter][LEXICAL_MAP_CLASSIFICATION] != CLASS_ALPHA)) throw new Exception("Cannot give the lowercase of a non-alpha.");
		if (theCharacter < 'a') return theCharacter + 32; 
		return theCharacter;  // already lower
	}
	
	/**
	 * Specific characters used by tools.
	 */
	public static final int		CRBYTEVALUE = 13;  		// ASCII BYTE VALUE
	public static final int		LFBYTEVALUE = 10;  		// ASCII BYTE VALUE
	public static final int		SPACEVALUE = 32;  		// ASCII BYTE VALUE
	public static final int		DASHVALUE = 45;  		// ASCII BYTE VALUE
	public static final int		DOLLARBYTEVALUE = 36; 	// ASCII BYTE VALUE
	public static final int		PIPEBYTEVALUE = 124;  	// ASCII BYTE VALUE
	public static final int		OPENBBYTEVALUE = 123;  	// ASCII BYTE VALUE
	public static final int		SLASHBYTEVALUE = 47; 	// ASCII BYTE VALUE
	public static final int		TABVALUE = 9; 			// ASCII BYTE VALUE
	public static final int		COLONVALUE = 58; 		// ASCII BYTE VALUE
	
	/**
	 * Sequences
	 */
	public static final String 	STRING_CRLF = "\r\n";
	
	/**
	 * Character classifications.
	 */
	public static final int CLASS_CONTROL = 0;
	public static final int CLASS_ALPHA = 1;
	public static final int CLASS_NUMERIC = 2;
	public static final int CLASS_PUNCTUATION = 3;
	public static final int CLASS_NONE = 4;
	
	/**
	 * Lexical names and types.
	 */
	public static final int	BAD = -1;
	public static final int	NOT_ALLOWED = -1;
	public static final int	NO_CHARACTER = -1;
	public static final int	ALLOWED = 400;
	
	public static final int	CONTROL = 300;
	public static final int	OTHER = 301;
	public static final int	BREAKING = 310;
	public static final int	SPECIAL = 311;
	
	public static final int	WS = 302;
	public static final int	WS_SPACE = 32;
	public static final int	WS_LF_CONTROL = 10;
	public static final int	WS_CR_CONTROL = 13;
	public static final int	WS_TAB_CONTROL = 9;

	public static final int CHAR = 303;
	public static final int	CHAR_DNSCHAR_POUND = 35;
	public static final int	CHAR_DNSCHAR_NUMERIC = 304;
	public static final int	CHAR_DNSCHAR = 305;
	
	public static final int	DNSCHAR = 312;
	public static final int URLCHAR	= 313;
	public static final int URLFCHAR	= 314;

	public static final int	SPECIAL_CHAR_DNSCHAR_DOT = 46;
	
	public static final int	SPECIAL_QUOTE = 34;
	public static final int	SPECIAL_PERCENT = 37;
	public static final int	SPECIAL_AMP	= 38;	
	public static final int	SPECIAL_OPENPAREN = 40;
	public static final int	SPECIAL_CLOSEPAREN = 41;
	public static final int	SPECIAL_SPLAT = 42;
	public static final int	SPECIAL_PLUS = 43;
	public static final int	SPECIAL_COMMA = 44;
	public static final int	SPECIAL_SLASH = 47;	
	public static final int	SPECIAL_COLON = 58;
	public static final int	SPECIAL_SEMICOLON = 59;
	public static final int	SPECIAL_LT = 60;
	public static final int	SPECIAL_EQ = 61;	
	public static final int	SPECIAL_GT = 62;
	public static final int	SPECIAL_QUEST = 63;
	public static final int	SPECIAL_AT = 64;
	public static final int	SPECIAL_OPENBRACK = 91;
	public static final int	SPECIAL_BACKSLASH = 92;
	public static final int	SPECIAL_CLOSEBRACK = 93;
	public static final int	SPECIAL_DOLLAR = 36;
	
	public static final int LEXICAL_HEADER_TERMINATION = SPECIAL_COLON;
	
	public static final int	ASCII_HIGH = 128;
	
	/**
	 * Values.
	 */
	public static final int VALUE_ASCII_LOW_BOTTOM	=	0;
	public static final int VALUE_ASCII_LOW_TOP		=	31;
	public static final int VALUE_ASCII_BOTTOM		=	32;
	public static final int VALUE_ASCII_TOP			=	127;	
	public static final int VALUE_ASCII_HIGH_BOTTOM	=	128;
	public static final int VALUE_ASCII_HIGH_TOP	=	255;	
	
	/**
	 * Lexical map.
	 */
	public static final int	LEXICAL_MAP_CLASSIFICATION  = 0;	// array column 1.
	public static final int	LEXICAL_MAP_NAME  = 1;				// array column 2.
	public static final int	LEXICAL_MAP_822_TYPE  = 2;			// array column 3.
	public static final int	LEXICAL_MAP_DNS_TYPE  = 3;			// array column 4.
	public static final int	LEXICAL_MAP_822_HEADERNAME  = 4;	// array column 5.
	public static final int	LEXICAL_MAP_URI_TYPE  = 5;			// array column 6.	
	public static final int	LEXICAL_MAP_HEXVALUE  = 6;			// array column 7.	
	public static final int	LEXICAL_MAP_URLF_TYPE  = 7;			// array column 8.	
	public static final int LEXICAL_MAP[][] = {
		{ CLASS_CONTROL,  CONTROL, 				CONTROL, 	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED }, // ^@	0	0	0	NUL	null
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^A	1	1	1	SOH	start of heading
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^B	2	2	2	STX	start of text
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^C	3	3	3	ETX	end of text
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^D	4	4	4	EOT	end of transmission
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^E	5	5	5	ENQ	enquiry
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^F	6	6	6	ACK	acknowledge
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^G	7	7	7	BEL	bell
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^H	8	8	10	BS	backspace
		{ CLASS_CONTROL,  WS_TAB_CONTROL,		WS,			BREAKING,		NOT_ALLOWED, 	WS, 			NOT_ALLOWED,	WS			}, // ^I	9	9	11	TAB	horizontal tab
		{ CLASS_CONTROL,  WS_LF_CONTROL,		WS,			BREAKING,		NOT_ALLOWED, 	WS_LF_CONTROL, 	NOT_ALLOWED,	WS_LF_CONTROL	}, // ^J	10	A	12	LF	new line
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^K	11	B	13	VT	vertical tab
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^L	12	C	14	FF	new page
		{ CLASS_CONTROL,  WS_CR_CONTROL,		WS,			BREAKING,		NOT_ALLOWED, 	WS_CR_CONTROL, 	NOT_ALLOWED,	WS_CR_CONTROL	}, // ^M	13	D	15	CR	carriage return
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^N	14	E	16	SO	shift out
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^O	15	F	17	SI	shift in
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^P	16	10	20	DLE	data link escape
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^Q	17	11	21	DC1	device CONTROL, 1
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^R	18	12	22	DC2	device CONTROL, 2
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^S	19	13	23	DC3	device CONTROL, 3
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^T	20	14	24	DC4	device CONTROL, 4
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^U	21	15	25	NAK	negative acknowledge
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^V	22	16	26	SYN	synchronous idle
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^W	23	17	27	ETB	end of trans. block
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^X	24	18	30	CAN	cancel
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^Y	25	19	31	EM	end of medium
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^Z	26	1A	32	SUB	substitute
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED }, // ^[	27	1B	33	ESC	escape
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^	28	1C	34	FS	file separator
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^]	29	1D	35	GS	group separator
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^^	30	1E	36	RS	record separator
		{ CLASS_CONTROL,  CONTROL,				CONTROL,	OTHER,			NOT_ALLOWED, 	NOT_ALLOWED, 	NOT_ALLOWED,	NOT_ALLOWED	}, // ^_	31	1F	37	US	unit separator
		{ CLASS_NONE,     WS_SPACE,				WS,			BREAKING,		NOT_ALLOWED,	WS, 			NOT_ALLOWED,	WS			}, // 		32	20	40	Space	space
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			NOT_ALLOWED,	OTHER,	 		NOT_ALLOWED,	URLFCHAR 	}, // 		33	21	41	!	
		{ CLASS_NONE,     SPECIAL_QUOTE,		SPECIAL,	OTHER,			NOT_ALLOWED,	OTHER, 			NOT_ALLOWED,	NOT_ALLOWED	}, // 		34	22	42	quote
		{ CLASS_NONE,     CHAR_DNSCHAR_POUND,	CHAR,		OTHER,			NOT_ALLOWED,	OTHER, 			NOT_ALLOWED,	URLFCHAR 	}, // 		35	23	43	#	
		{ CLASS_NONE,     CHAR_DNSCHAR,			CHAR_DNSCHAR,	DNSCHAR,	NOT_ALLOWED,	OTHER, 			NOT_ALLOWED,	URLFCHAR  	}, // 		36	24	44	$	
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			NOT_ALLOWED,	SPECIAL_PERCENT,NOT_ALLOWED,	SPECIAL_PERCENT	}, // 	37	25	45	%	
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			NOT_ALLOWED,	SPECIAL_AMP,	NOT_ALLOWED,	SPECIAL_AMP	}, // 		38	26	46	&	
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			NOT_ALLOWED,	OTHER,	 		NOT_ALLOWED,	URLFCHAR	}, // 		39	27	47	'	
		{ CLASS_NONE,     SPECIAL_OPENPAREN,	SPECIAL,	BREAKING,		NOT_ALLOWED,	OTHER, 			NOT_ALLOWED,	URLFCHAR	}, // 		40	28	50	(	
		{ CLASS_NONE,     SPECIAL_CLOSEPAREN,	SPECIAL,	BREAKING,		NOT_ALLOWED,	OTHER, 			NOT_ALLOWED,	URLFCHAR	}, // 		41	29	51	)	
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			NOT_ALLOWED,	SPECIAL_SPLAT, 	NOT_ALLOWED,	URLFCHAR	}, // 		42	2A	52	* (splat)	
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			NOT_ALLOWED,	SPECIAL_PLUS, 	NOT_ALLOWED,	SPECIAL_PLUS }, // 		43	2B	53	+	
		{ CLASS_NONE,     SPECIAL_COMMA,		SPECIAL,	BREAKING,		NOT_ALLOWED,	OTHER, 			NOT_ALLOWED,	URLFCHAR	}, // 		44	2C	54	,	
		{ CLASS_NONE,     CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR, 		NOT_ALLOWED,	URLFCHAR	}, // 		45	2D	55	-	
		{ CLASS_NONE,     SPECIAL_CHAR_DNSCHAR_DOT,	CHAR,	DNSCHAR,		ALLOWED,		URLCHAR, 		NOT_ALLOWED,	URLFCHAR	}, // 		46	2E	56	.	
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			ALLOWED, 		SPECIAL_SLASH,  NOT_ALLOWED,	URLFCHAR	}, // 		47	2F	57	/	slash
		{ CLASS_NONE,     CHAR_DNSCHAR_NUMERIC,	CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		0,				URLFCHAR	}, // 		48	30	60	0	
		{ CLASS_NUMERIC,  CHAR_DNSCHAR_NUMERIC,	CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		1,				URLFCHAR	}, // 		49	31	61	1	
		{ CLASS_NUMERIC,  CHAR_DNSCHAR_NUMERIC,	CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		2,				URLFCHAR	}, // 		50	32	62	2	
		{ CLASS_NUMERIC,  CHAR_DNSCHAR_NUMERIC,	CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		3,				URLFCHAR	}, // 		51	33	63	3	
		{ CLASS_NUMERIC,  CHAR_DNSCHAR_NUMERIC,	CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		4,				URLFCHAR	}, // 		52	34	64	4	
		{ CLASS_NUMERIC,  CHAR_DNSCHAR_NUMERIC,	CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		5,				URLFCHAR	}, // 		53	35	65	5	
		{ CLASS_NUMERIC,  CHAR_DNSCHAR_NUMERIC,	CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		6,				URLFCHAR	}, // 		54	36	66	6	
		{ CLASS_NUMERIC,  CHAR_DNSCHAR_NUMERIC,	CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		7,				URLFCHAR	}, // 		55	37	67	7	
		{ CLASS_NUMERIC,  CHAR_DNSCHAR_NUMERIC,	CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		8,				URLFCHAR	}, // 		56	38	70	8	
		{ CLASS_NUMERIC,  CHAR_DNSCHAR_NUMERIC,	CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		9,				URLFCHAR	}, // 		57	39	71	9	
		{ CLASS_NONE,     SPECIAL_COLON,		SPECIAL,	BREAKING,		BREAKING,		SPECIAL_COLON,  		NOT_ALLOWED,	URLFCHAR	}, // 		58	3A	72	:	colon
		{ CLASS_NONE,     SPECIAL_SEMICOLON,	SPECIAL,	BREAKING,		ALLOWED,		OTHER,  		NOT_ALLOWED,	URLFCHAR	}, // 		59	3B	73	;	semicolon
		{ CLASS_NONE,     SPECIAL_LT,			SPECIAL,	BREAKING,		ALLOWED,		OTHER,  		NOT_ALLOWED,	NOT_ALLOWED	}, // 	.	60	3C	74	<	
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			ALLOWED,		SPECIAL_EQ,  	NOT_ALLOWED,	SPECIAL_EQ	}, // 		61	3D	75	=	
		{ CLASS_NONE,     SPECIAL_GT,			SPECIAL,	BREAKING,		ALLOWED,		OTHER,  		NOT_ALLOWED,	NOT_ALLOWED	}, // 	.	62	3E	76	>	
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			ALLOWED,		SPECIAL_QUEST,	NOT_ALLOWED,	URLFCHAR	}, // 		63	3F	77	?
		{ CLASS_NONE,     SPECIAL_AT,			SPECIAL,	OTHER,			ALLOWED,		OTHER,  		NOT_ALLOWED,	URLFCHAR	}, // 	.	64	40	100	@
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		10,				URLFCHAR	}, // 		65	41	101	A
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		11,				URLFCHAR	}, // 		66	42	102	B
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		12,				URLFCHAR	}, //	 	67	43	103	C
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		13,				URLFCHAR	}, //	 	68	44	104	D
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		14,				URLFCHAR	}, // 		69	45	105	E
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,		15,				URLFCHAR	}, // 		70	46	106	F
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		71	47	107	G
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		72	48	110	H
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, //	 	73	49	111	I
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, //	 	74	4A	112	J
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, //	 	75	4B	113	K
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, //	 	76	4C	114	L
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		77	4D	115	M
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		78	4E	116	N
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		79	4F	117	O
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		80	50	120	P
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, //		81	51	121	Q
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		82	52	122	R
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		83	53	123	S
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		84	54	124	T
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		85	55	125	U
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		86	56	126	V
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		87	57	127	W
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		88	58	130	X
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		89	59	131	Y
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		90	5A	132	Z
		{ CLASS_NONE,     SPECIAL_OPENBRACK,	SPECIAL,	OTHER,			ALLOWED,		OTHER,  		NOT_ALLOWED,	URLFCHAR	}, // 		91	5B	133	[
		{ CLASS_NONE,     SPECIAL_BACKSLASH,	SPECIAL,	OTHER,			ALLOWED,		OTHER,  		NOT_ALLOWED,	NOT_ALLOWED	}, // 		92	5C	134	\
		{ CLASS_NONE,     SPECIAL_CLOSEBRACK,	SPECIAL,	OTHER,			ALLOWED,		OTHER,  		NOT_ALLOWED,	URLFCHAR	}, //	 	93	5D	135	]
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			ALLOWED,		OTHER,  		NOT_ALLOWED,	NOT_ALLOWED	}, // 		94	5E	136	^
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, //	 	95	5F	137	_
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			ALLOWED,		URLCHAR,  		NOT_ALLOWED,	NOT_ALLOWED	}, // 		96	60	140	`
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		10,				URLFCHAR	}, // 		97	61	141	a
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		11,				URLFCHAR	}, // 		98	62	142	b
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		12,				URLFCHAR	}, //	 	99	63	143	c
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		13,				URLFCHAR	}, // 		100	64	144	d
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		14,				URLFCHAR	}, // 		101	65	145	e
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		15,				URLFCHAR	}, // 		102	66	146	f
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		103	67	147	g
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		104	68	150	h
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		105	69	151	i
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		106	6A	152	j
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, //	 	107	6B	153	k
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		108	6C	154	l
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		109	6D	155	m
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		110	6E	156	n
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		111	6F	157	o
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		112	70	160	p
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		113	71	161	q
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		114	72	162	r
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		115	73	163	s
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		116	74	164	t
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		117	75	165	u
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		118	76	166	v
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		119	77	167	w
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, //	 	120	78	170	x
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		121	79	171	y
		{ CLASS_ALPHA,    CHAR_DNSCHAR,			CHAR,		DNSCHAR,		ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		122	7A	172	z
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			ALLOWED,		OTHER,  		NOT_ALLOWED,	NOT_ALLOWED	}, // 		123	7B	173	{
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			ALLOWED,		OTHER,  		NOT_ALLOWED,	NOT_ALLOWED	}, // 		124	7C	174	
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			ALLOWED,		OTHER,  		NOT_ALLOWED,	NOT_ALLOWED	}, // 		125	7D	175	}
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			ALLOWED,		URLCHAR,  		NOT_ALLOWED,	URLFCHAR	}, // 		126	7E	176	~
		{ CLASS_NONE,     CHAR,					CHAR,		OTHER,			ALLOWED,		OTHER,  		NOT_ALLOWED,	NOT_ALLOWED	}, // 		127	7F	177	DEL
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 		128			
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	129			
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	130			
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	131			
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	132			
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	133			
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	134			
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	135			
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	136			
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	137			
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	138			
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	139			
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	140			
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	141			
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	142			
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	143
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	144
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	145
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	146
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	147
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	148
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	149
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	150
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	151
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	152
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	153
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	154
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	155
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	156
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	157
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	158
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	159
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	160
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	161
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	162
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	163
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	164
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	165
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	166
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	167
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	168
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	169
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	170
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	171
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	172
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	173
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	174
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	175
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	176
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	177
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	178
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	179
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	180
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	181
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	182
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	183
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	184
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	185
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	186
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	187
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	188
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	189
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	190
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	191
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	192
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	193
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	194
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	195
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	196
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	197
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	198
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	199
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	200
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	201
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	202
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	203
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	204
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	205
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	206
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	207
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	208
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	209
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	210
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	211
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	212
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	213
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	214
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	215
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	216
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	217
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	218
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	219
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	220
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	221
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	222
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	223
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	224
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	225
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	226
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	227
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	228
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	229
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	230
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	231
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	232
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	233
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	234
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	235
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	236
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	237
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	238
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	239
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	240
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	241
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	242
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	243
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	244
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	245
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	246
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	247
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	248
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	249
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	250
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	251
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	252
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	253
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}, // 	254
		{ CLASS_NONE,     ASCII_HIGH,			ASCII_HIGH,	OTHER,			NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED,	NOT_ALLOWED	}  // 	255
	};
	
	/**
	 * Header processing chart.<p>
<pre>  
0	HP_START			clear_flags	read	transition		
1	HP_BROKEN			transition		FLAG_BROKEN
2	HP_READ				yield	read	transition		
3	HP_HEAD_CR			yield	read	transition		
4	HP_HEAD_LF			yield	read	transition		
5	HP_CR				yield	read	transition		
6	HP_LF				yield	read	transition		
7	HP_LFCR				yield	read	transition		
8	HP_PAUSE			transition		PAUSE
9	HP_CLEAR_PAUSE		clear_flags		transition		
10	HP_PAUSE_CRLF		transition		PAUSE
11	HP_CLEAR_PAUSE_CRLF	clear_flags		transition		
12	HP_PAUSE_CRLFCR		yield	read	transition		PAUSE
13	HP_HEAD_CRLF		yield	read	transition		
X	HP_CLOSURE			yield				DONE
</pre>
	 */
	public final static int		HP_NOT_USED = -1;
	public final static int 	HP_START = 0;
	public final static int 	HP_BROKEN = 1;
	public final static int 	HP_READ = 2;
	public final static int 	HP_HEAD_CR = 3;
	public final static int 	HP_HEAD_LF = 4;
	public final static int 	HP_CR = 5;
	public final static int 	HP_LF  = 6;
	public final static int 	HP_LFCR = 7;
	public final static int 	HP_PAUSE = 8;
	public final static int 	HP_CLEAR_PAUSE = 9;
	public final static int 	HP_PAUSE_CRLF = 10;
	public final static int 	HP_CLEAR_PAUSE_CRLF = 11;
	public final static int 	HP_PAUSE_CRLFCR = 12;
	public final static int 	HP_HEAD_CRLF = 13;
	public final static int 	HP_CLOSURE = 14;
	
	public final static int 	HP____SPECIAL_PAUSE = 100;
	public final static int 	HP____SPECIAL_WALKING_DEAD = 101;
	public final static int 	HP____SPECIAL_DEAD = 102;
	
	public static final int HEADER_READ_STATE_CHART[][] = {
		//HP_START		HP_BROKEN	HP_READ		HP_HEAD_CR	HP_HEAD_LF  HP_CR		HP_LF		HP_LFCR		HP_PAUSE		HP_CLEAR_PAUSE 		HP_PAUSE_CRLF	HP_CLEAR_PAUSE_CRLF	HP_PAUSE_CRLFCR	HP_HEAD_CRLF,   
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^@	CTRL	0	0	0	NUL	null
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^A	CTRL	1	1	1	SOH	start of heading
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^B	CTRL	2	2	2	STX	start of text
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^C	CTRL	3	3	3	ETX	end of text
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^D	CTRL	4	4	4	EOT	end of transmission
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^E	CTRL	5	5	5	ENQ	enquiry
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^F	CTRL	6	6	6	ACK	acknowledge
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^G	CTRL	7	7	7	BEL	bell
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^H	CTRL	8	8	10	BS	backspace
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_READ,	HP_READ,	HP_BROKEN,	HP_BROKEN,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_READ	}	,	// ^I	WS	9	9	11	TAB	horizontal tab
		{ HP_LF,		HP_READ,	HP_HEAD_LF,	HP_HEAD_CRLF,HP_CLOSURE,HP_CLOSURE,	HP_CLOSURE,	HP_CLOSURE,	HP_CLEAR_PAUSE,	HP_LF,		HP_CLEAR_PAUSE_CRLF,	HP_CLOSURE,			HP_CLOSURE	,	HP_CLOSURE	}	,	// ^J	LF	10	A	12	LF	new line
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^K	CTRL	11	B	13	VT	vertical tab
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^L	CTRL	12	C	14	FF	new page
		{ HP_CR,		HP_READ,	HP_HEAD_CR,	HP_CLOSURE,	HP_LFCR,	HP_CLOSURE,	HP_LFCR,	HP_CLOSURE,	HP_CLEAR_PAUSE,	HP_CR,		HP_CLEAR_PAUSE_CRLF,	HP_PAUSE_CRLFCR,	HP_CLOSURE	,	HP_PAUSE_CRLFCR	}	,	// ^M	CR	13	D	15	CR	carriage return
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_LF,		HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^N	CTRL	14	E	16	SO	shift out
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^O	CTRL	15	F	17	SI	shift in
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^P	CTRL	16	10	20	DLE	data link escape
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^Q	CTRL	17	11	21	DC1	device CONTROL, 1
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^R	CTRL	18	12	22	DC2	device CONTROL, 2
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^S	CTRL	19	13	23	DC3	device CONTROL, 3
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^T	CTRL	20	14	24	DC4	device CONTROL, 4
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^U	CTRL	21	15	25	NAK	negative acknowledge
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^V	CTRL	22	16	26	SYN	synchronous idle
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^W	CTRL	23	17	27	ETB	end of trans. block
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^X	CTRL	24	18	30	CAN	cancel
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^Y	CTRL	25	19	31	EM	end of medium
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^Z	CTRL	26	1A	32	SUB	substitute
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^[	CTRL	27	1B	33	ESC	escape
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^	CTRL	28	1C	34	FS	file separator
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^]	CTRL	29	1D	35	GS	group separator
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^^	CTRL	30	1E	36	RS	record separator
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^_	CTRL	31	1F	37	US	unit separator
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_READ,	HP_READ,	HP_BROKEN,	HP_BROKEN,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_READ	}	,	// 	WS	32	20	40	Space	space
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	33	21	41	!	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	34	22	42	.	quote (double)
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	35	23	43	#	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	36	24	44	$	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	37	25	45	%	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	38	26	46	&	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	39	27	47	'	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	40	28	50	(	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	41	29	51	)	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	42	2A	52	*	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	43	2B	53	+	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	44	2C	54	,	comma
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	45	2D	55	-	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	46	2E	56	.	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	//	CHAR	47	2F	57	/	slash
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	48	30	60	0	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	49	31	61	1	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	50	32	62	2	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	51	33	63	3	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	52	34	64	4	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	53	35	65	5	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	54	36	66	6	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	55	37	67	7	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	56	38	70	8	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	57	39	71	9	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	58	3A	72	:	colon
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	59	3B	73	;	semicolon
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// .	CHAR	60	3C	74	<	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	61	3D	75	=	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// .	CHAR	62	3E	76	>	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	63	3F	77	?	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// .	CHAR	64	40	100	@	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	65	41	101	A	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	66	42	102	B	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	67	43	103	C	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	68	44	104	D	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	69	45	105	E	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	70	46	106	F	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	71	47	107	G	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	72	48	110	H	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	73	49	111	I	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	74	4A	112	J	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	75	4B	113	K	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	76	4C	114	L	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	77	4D	115	M	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	78	4E	116	N	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	79	4F	117	O	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	80	50	120	P	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	81	51	121	Q	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	82	52	122	R	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	83	53	123	S	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	84	54	124	T	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	85	55	125	U	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	86	56	126	V	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	87	57	127	W	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	88	58	130	X	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	89	59	131	Y	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	90	5A	132	Z	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	91	5B	133	[	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	92	5C	134	\	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	93	5D	135	]	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	94	5E	136	^	carot
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	95	5F	137	_	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	96	60	140	`	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	97	61	141	a	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	98	62	142	b	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	99	63	143	c	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	100	64	144	d	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	101	65	145	e	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	102	66	146	f	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	103	67	147	g	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	104	68	150	h	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	105	69	151	i	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	106	6A	152	j	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	107	6B	153	k	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	108	6C	154	l	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	109	6D	155	m	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	110	6E	156	n	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	111	6F	157	o	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	112	70	160	p	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	113	71	161	q	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	114	72	162	r	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	115	73	163	s	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	116	74	164	t	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	117	75	165	u	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	118	76	166	v	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	119	77	167	w	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	120	78	170	x	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	121	79	171	y	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	122	7A	172	z	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	123	7B	173	{ 	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	124	7C	174		
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	125	7D	175	}	
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	126	7E	176	~	
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	127	7F	177	DEL	
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	128				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	129				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	130				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	131				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	132				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	133				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	134				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	135				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	136				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	137				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	138				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	139				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	140				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	141				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	142				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	143				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	144				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	145				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	146				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	147				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	148				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	149				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	150				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	151				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	152				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	153				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	154				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	155				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	156				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	157				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	158				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	159				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	160				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	161				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	162				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	163				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	164				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	165				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	166				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	167				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	168				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	169				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	170				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	171				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	172				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	173				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	174				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	175				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	176				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	177				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	178				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	179				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	180				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	181				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	182				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	183				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	184				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	185				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	186				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	187				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	188				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	189				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	190				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	191				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	192				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	193				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	194				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	195				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	196				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	197				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	198				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	199				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	200				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	201				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	202				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	203				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	204				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	205				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	206				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	207				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	208				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	209				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	210				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	211				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	212				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	213				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	214				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	215				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	216				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	217				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	218				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	219				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	220				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	221				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	222				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	223				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	224				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	225				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	226				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	227				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	228				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	229				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	230				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	231				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	232				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	233				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	234				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	235				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	236				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	237				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	238				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	239				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	240				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	241				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	242				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	243				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	244				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	245				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	246				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	247				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	248				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	249				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	250				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	251				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	252				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	253				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	254				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	255				
	
	};

	public static final int HEADER_READ_STATE_CHARTV2[][] = {
		//HP_START		HP_BROKEN	HP_READ		HP_HEAD_CR	HP_HEAD_LF  HP_CR		HP_LF		HP_LFCR		HP_PAUSE		HP_CLEAR_PAUSE 		HP_PAUSE_CRLF	HP_CLEAR_PAUSE_CRLF	HP_PAUSE_CRLFCR	HP_HEAD_CRLF,   
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^@	CTRL	0	0	0	NUL	null
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^A	CTRL	1	1	1	SOH	start of heading
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^B	CTRL	2	2	2	STX	start of text
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^C	CTRL	3	3	3	ETX	end of text
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^D	CTRL	4	4	4	EOT	end of transmission
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^E	CTRL	5	5	5	ENQ	enquiry
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^F	CTRL	6	6	6	ACK	acknowledge
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^G	CTRL	7	7	7	BEL	bell
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^H	CTRL	8	8	10	BS	backspace
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_READ,	HP_READ,	HP_BROKEN,	HP_BROKEN,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_READ	}	,	// ^I	WS	9	9	11	TAB	horizontal tab
		{ HP_LF,		HP_READ,	HP_HEAD_LF,	HP_HEAD_CRLF,HP_PAUSE_CRLFCR,HP_PAUSE_CRLFCR,	HP_PAUSE_CRLFCR,	HP_CLOSURE,	HP_CLEAR_PAUSE,	HP_LF,		HP_CLEAR_PAUSE_CRLF,	HP_CLOSURE,			HP_CLOSURE	,	HP_CLOSURE	}	,	// ^J	LF	10	A	12	LF	new line
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^K	CTRL	11	B	13	VT	vertical tab
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^L	CTRL	12	C	14	FF	new page
		{ HP_CR,		HP_READ,	HP_HEAD_CR,	HP_CLOSURE,	HP_LFCR,	HP_PAUSE_CRLFCR,	HP_PAUSE_CRLFCR,	HP_CLOSURE,	HP_CLEAR_PAUSE,	HP_CR,		HP_CLEAR_PAUSE_CRLF,	HP_PAUSE_CRLFCR,	HP_CLOSURE	,	HP_PAUSE_CRLFCR	}	,	// ^M	CR	13	D	15	CR	carriage return
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_LF,		HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^N	CTRL	14	E	16	SO	shift out
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^O	CTRL	15	F	17	SI	shift in
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^P	CTRL	16	10	20	DLE	data link escape
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^Q	CTRL	17	11	21	DC1	device CONTROL, 1
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^R	CTRL	18	12	22	DC2	device CONTROL, 2
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^S	CTRL	19	13	23	DC3	device CONTROL, 3
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^T	CTRL	20	14	24	DC4	device CONTROL, 4
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^U	CTRL	21	15	25	NAK	negative acknowledge
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^V	CTRL	22	16	26	SYN	synchronous idle
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^W	CTRL	23	17	27	ETB	end of trans. block
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^X	CTRL	24	18	30	CAN	cancel
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^Y	CTRL	25	19	31	EM	end of medium
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^Z	CTRL	26	1A	32	SUB	substitute
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^[	CTRL	27	1B	33	ESC	escape
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^	CTRL	28	1C	34	FS	file separator
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^]	CTRL	29	1D	35	GS	group separator
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^^	CTRL	30	1E	36	RS	record separator
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// ^_	CTRL	31	1F	37	US	unit separator
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_READ,	HP_READ,	HP_BROKEN,	HP_BROKEN,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,		HP_PAUSE	,	HP_READ	}	,	// 	WS	32	20	40	Space	space
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	33	21	41	!	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	34	22	42	.	quote (double)
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	35	23	43	#	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	36	24	44	$	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	37	25	45	%	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	38	26	46	&	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	39	27	47	'	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	40	28	50	(	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	41	29	51	)	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	42	2A	52	*	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	43	2B	53	+	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	44	2C	54	,	comma
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	45	2D	55	-	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	46	2E	56	.	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	//	CHAR	47	2F	57	/	slash
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	48	30	60	0	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	49	31	61	1	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	50	32	62	2	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	51	33	63	3	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	52	34	64	4	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	53	35	65	5	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	54	36	66	6	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	55	37	67	7	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	56	38	70	8	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	57	39	71	9	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	58	3A	72	:	colon
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	59	3B	73	;	semicolon
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// .	CHAR	60	3C	74	<	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	61	3D	75	=	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// .	CHAR	62	3E	76	>	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	63	3F	77	?	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// .	CHAR	64	40	100	@	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	65	41	101	A	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	66	42	102	B	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	67	43	103	C	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	68	44	104	D	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	69	45	105	E	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	70	46	106	F	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	71	47	107	G	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	72	48	110	H	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	73	49	111	I	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	74	4A	112	J	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	75	4B	113	K	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	76	4C	114	L	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	77	4D	115	M	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	78	4E	116	N	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	79	4F	117	O	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	80	50	120	P	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	81	51	121	Q	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	82	52	122	R	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	83	53	123	S	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	84	54	124	T	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	85	55	125	U	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	86	56	126	V	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	87	57	127	W	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	88	58	130	X	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	89	59	131	Y	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	90	5A	132	Z	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	91	5B	133	[	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	92	5C	134	\	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	93	5D	135	]	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	94	5E	136	^	carot
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	95	5F	137	_	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	96	60	140	`	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	97	61	141	a	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	98	62	142	b	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	99	63	143	c	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	100	64	144	d	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	101	65	145	e	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	102	66	146	f	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	103	67	147	g	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	104	68	150	h	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	105	69	151	i	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	106	6A	152	j	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	107	6B	153	k	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	108	6C	154	l	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	109	6D	155	m	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	110	6E	156	n	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	111	6F	157	o	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	112	70	160	p	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	113	71	161	q	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	114	72	162	r	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	115	73	163	s	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	116	74	164	t	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	117	75	165	u	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	118	76	166	v	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	119	77	167	w	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	120	78	170	x	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	121	79	171	y	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	122	7A	172	z	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	123	7B	173	{ 	
		{ HP_READ,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	124	7C	174		
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	125	7D	175	}	
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	126	7E	176	~	
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_READ,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	CHAR	127	7F	177	DEL	
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	128				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	129				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	130				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	131				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	132				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	133				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	134				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	135				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	136				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	137				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	138				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	139				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	140				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	141				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	142				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	143				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	144				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	145				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	146				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	147				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	148				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	149				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	150				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	151				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	152				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	153				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	154				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	155				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	156				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	157				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	158				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	159				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	160				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	161				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	162				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	163				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	164				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	165				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	166				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	167				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	168				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	169				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	170				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	171				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	172				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	173				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	174				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	175				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	176				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	177				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	178				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	179				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	180				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	181				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	182				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	183				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	184				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	185				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	186				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	187				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	188				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	189				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	190				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	191				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	192				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	193				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	194				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	195				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	196				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	197				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	198				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	199				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	200				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	201				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	202				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	203				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	204				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	205				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	206				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	207				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	208				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	209				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	210				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	211				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	212				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	213				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	214				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	215				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	216				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	217				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	218				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	219				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	220				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	221				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	222				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	223				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	224				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	225				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	226				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	227				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	228				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	229				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	230				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	231				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	232				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	233				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	234				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	235				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	236				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	237				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	238				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	239				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	240				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	241				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	242				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	243				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	244				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	245				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	246				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	247				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	248				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	249				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	250				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	251				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	252				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	253				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	254				
		{ HP_BROKEN,	HP_READ,	HP_READ,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_PAUSE,	HP_CLEAR_PAUSE,	HP_BROKEN,	HP_CLEAR_PAUSE_CRLF,	HP_CLEAR_PAUSE,	HP_PAUSE	,	HP_PAUSE_CRLF	}	,	// 	UPPER	255				
	
	};
	
}


